--[[
 * ReaScript Name: Import markers and regions from tab-delimited CSV file
 * Description: See title.
 * Instructions: Select a track. Run.
 * Author: X-Raym
 * Author URI: http://www.extremraym.com
 * Repository: GitHub > X-Raym > EEL Scripts for Cockos REAPER
 * Repository URI: https://github.com/X-Raym/REAPER-EEL-Scripts
 * Links
    Forum Thread https://forum.cockos.com/showthread.php?p=1670961
 * Licence: GPL v3
 * REAPER: 5.0
 * Version: 1.0
--]]

--[[
 * Changelog:
 * v1.0 (2019-01-26)
  + Initial Release
--]]

-- USER CONFIG AREA -----------------------------------------------------------
-- Duplicate and Rename the script if you want to modify this.
-- Else, a script update will erase your mods.

-- Display a message in the console for debugging
function Msg(value)
  if console then
    reaper.ShowConsoleMsg(tostring(value) .. "\n")
  end
end

console = true -- true/false: display debug messages in the console
sep = "," -- default sep

col_pos = 7 -- Position column index in the CSV
col_pos_end = 4 -- Length column index in the CS
col_len = 6 -- Length column index in the CSV
col_name = 2 -- Name column index in the CSV
col_color = 3
col_pattern = 1
col_ticks = 5

unit = 4
bpm, beat_per_measure = reaper.GetProjectTimeSignature2(0)
Msg("BPM ".. bpm)

------------------------------------------------------- END OF USER CONFIG AREA

function ColorHexToInt(hex)
  hex = hex:gsub("#", "")
  local R = tonumber("0x"..hex:sub(1,2))
  local G = tonumber("0x"..hex:sub(3,4))
  local B = tonumber("0x"..hex:sub(5,6))
  return reaper.ColorToNative(R, G, B)
end

-- Optimization
local reaper = reaper

-- CSV to Table
-- http://lua-users.org/wiki/LuaCsv
function ParseCSVLine (line,sep)
  local line = string.gsub(line, "{.*}", "")
  local res = {}
  local pos = 1 
  sep = sep or ','
  while true do
    local c = string.sub(line,pos,pos)
    if (c == "") then break end
    if (c == '"') then
      -- quoted value (ignore separator within)
      local txt = ""
      repeat
        local startp,endp = string.find(line,'^%b""',pos)
        txt = txt..string.sub(line,startp+1,endp-1)
        pos = endp + 1
        c = string.sub(line,pos,pos)
        if (c == '"') then txt = txt..'"' end
        -- check first char AFTER quoted string, if it is another
        -- quoted string without separator, then append it
        -- this is the way to "escape" the quote char in a quote. example:
        --   value1,"blub""blip""boing",value3  will result in blub"blip"boing  for the middle
      until (c ~= '"')
      table.insert(res,txt)
      assert(c == sep or c == "")
      pos = pos + 3
    else
      -- no quotes used, just look for the first separator
      local startp,endp = string.find(line,sep,pos)
      if (startp) then
        table.insert(res,string.sub(line,pos,startp-1))
        pos = endp + 1
      else
        -- no separator found -> use rest of string and terminate
        table.insert(res,string.sub(line,pos))
        break
      end
    end
  end
  return res
end


-- UTILITIES -------------------------------------------------------------

-- Display a message in the console for debugging
function Msg(value)
  if console then
    reaper.ShowConsoleMsg(tostring(value) .. "\n")
  end
end

function ReverseTable(t)
    local reversedTable = {}
    local itemCount = #t
    for k, v in ipairs(t) do
        reversedTable[itemCount + 1 - k] = v
    end
    return reversedTable
end

--------------------------------------------------------- END OF UTILITIES

function read_lines(filepath)

  lines = {}

  local f = io.input(filepath)
  repeat

    s = f:read ("*l") -- read one line

    if s then  -- if not end of file (EOF)
      table.insert(lines, ParseCSVLine (s,sep))
    end

  until not s  -- until end of file

  f:close()

end


-- Main function
function main()

  folder = filetxt:match[[^@?(.*[\/])[^\/]-$]]

  for i, line in ipairs( lines ) do
    if line[col_pattern] == "pattern" then 
   
     if i > 1 then
      
      -- Name Variables
      local pos = line[col_pos]
      --local pos_end = tonumber(line[col_pos] +1) --tonumber(line[col_pos_end])
      local len =  line[col_len] 
      local name = line[col_name]
      local color = 0
      if line[col_name] == "Count-in" then
        color = ColorHexToInt("#FF80C0")|0x1000000
        duration = 2
      end      
      if line[col_color] == "A" and line[col_name] == "PostFill" then
        color = ColorHexToInt("#1127AE")|0x1000000
        duration = 1
      end
      if line[col_color] == "A" and line[col_name] == "Normal" then
        color = ColorHexToInt("#3776EB")|0x1000000
        duration = 1
      end
      if line[col_color] == "A" and line[col_name] == "Fill" then
        color = ColorHexToInt("#71BEF1")|0x1000000
        duration = 1
      end
      if line[col_color] == "Aending" and line[col_name] == "ending" then
        color = ColorHexToInt("#97D0FE")|0x1000000
        duration = 2
      end      
      if line[col_color] == "B" and line[col_name] == "PostFill" then
        color = ColorHexToInt("#0B7427")|0x1000000
        duration = 1
      end
      if line[col_color] == "B" and line[col_name] == "Normal" then
        color = ColorHexToInt("#11AE3B")|0x1000000
        length = 1
      end
      if line[col_color] == "B" and line[col_name] == "Fill" then
        color = ColorHexToInt("#50ED7B")|0x1000000
        duration = 1
      end      
      if line[col_color] == "Bending" and line[col_name] == "ending" then
        color = ColorHexToInt("#9EFEB6")|0x1000000
        duration = 2
      end
      
      
        
        pos2 = ((((pos+1) * beat_per_measure)/ bpm) * 60)  -- +2 bar offset count-in
        --len2 = (((len * beat_per_measure)/ bpm) * 60)
        len2 = (((duration * beat_per_measure)/ bpm) * 60)
        reaper.AddProjectMarker2( 0, true, pos2,pos2 + len2, name, -1, color ) 
      end
      end
    end
      
end

function main2()

  folder = filetxt:match[[^@?(.*[\/])[^\/]-$]]

  for i, line in ipairs( lines ) do
  
    bug_pattern = "Shot{ Kick (applicable for Brushes & Sticks)  },1,320,0,725,1,90"
    
    line_pattern = line[col_pattern]
    --shot_pattern = string.gsub(line_pattern,"{.*}","")  str = str:gsub('%b()', '')
    shot_pattern = string.gsub(bug_pattern, "{.*}", "")
   -- Msg("Bug ".. shot_pattern)
    --Msg("Line pattern ".. line_pattern)
    --Msg("Shot pattern ".. shot_pattern)
    
    --if shot_pattern  == "Shot" then
    if line_pattern  == "Shot" then
    Msg("Line ".. line_pattern)
    
     
    --Msg("Is Shot ")
     if i > 1 then
      
      -- Name Variables
      local pos = line[col_color] -- bar is column 2 for Shots
      --local pos_end = tonumber(line[col_pos] +1) --tonumber(line[col_pos_end])
      local len =  line[col_len] 
      local name = line[col_name]
      local color = 0
      local ticks = line[col_ticks]
      local dura_ticks = line[col_pos_end]
      if line[col_name] == "0" then
        color = ColorHexToInt("#BFBFBF")|0x1000000 -- Hold
        
      end
      if line[col_name] == "1" then
        color = ColorHexToInt("#7F7F7F")|0x1000000 -- Shot
        
      end
        --ticks_pos = (dura_ticks / 220)
        --duration = (ticks /220)
        ticks_pos = (dura_ticks / (bpm *2))
        duration = (ticks / (bpm *2))

        
        
        Msg("============")
        Msg("BPM ".. bpm)
        Msg("Pos ".. pos+2)        
        Msg("Druation ticks ".. dura_ticks)
        Msg("Duration ".. duration)
        Msg("Ticks ".. ticks)
        Msg("Ticks pos ".. ticks_pos)
        
        
        --pos1 = string.gsub(pos,"{(.*)}","")    
        --Msg("Pos " .. pos)
        pos2 = ((((pos+1) * beat_per_measure)/ bpm) * 60) --+ 2 -- +_2 bar offset count-in
        --len2 = (((len * beat_per_measure)/ bpm) * 60)
        --len2 =  (((duration * beat_per_measure)/ bpm) * 60)
        
        --Msg("Duration ".. duration)
        reaper.AddProjectMarker2( 0, true, pos2 + ticks_pos, pos2 + duration, name, -1, color ) 
      end
      end
    end
      
end




-- INIT

retval, filetxt = reaper.GetUserFileNameForRead("", "Import markers and regions", "txt")

if retval then

  reaper.PreventUIRefresh(1)

  reaper.Undo_BeginBlock() -- Begining of the undo block. Leave it at the top of your main function.
  
  reaper.ClearConsole()

  read_lines(filetxt)
  
  -- reaper.Main_OnCommand( reaper.NamedCommandLookup( "_SWSMARKERLIST10" ), -1) -- SWS: Delete all regions

  main()
  
  main2()

  reaper.Undo_EndBlock("Import markers and regions from tab-delimited txt", -1) -- End of the undo block. Leave it at the bottom of your main function.

  reaper.UpdateArrange()

  reaper.PreventUIRefresh(-1)

end
