

-- USER CONFIG AREA -----------------------------------------------------------

console = true -- true/false: display debug messages in the console

replace_name = true -- Replace items names by region names from CSV

delete_items = true -- Delete items outside regions

------------------------------------------------------- END OF USER CONFIG AREA

-- Note: Use it on items that have original edges and rate 1.
-- Note: No CSV validation

-- UTILITIES -------------------------------------------------------------

-- Save item selection
function SaveSelectedItems (table)
  for i = 0, reaper.CountSelectedMediaItems(0)-1 do
    table[i+1] = reaper.GetSelectedMediaItem(0, i)
  end
end

function RestoreSelectedItems (table)
  for _, item in ipairs(table) do
    reaper.SetMediaItemSelected(item, true)
  end
end

-- Display a message in the console for debugging
function Msg(value)
  if console then
    reaper.ShowConsoleMsg(tostring(value) .. "\n")
  end
end

-- Count the number of times a value occurs in a table
function CountOccurencesTable(tt, item)
  local count = 0
  for ii,xx in pairs(tt) do
    if item == xx then count = count + 1 end
  end
  return count
end -- CountOccurencesTable()

-- Remove duplicates from a table array
function RemoveTableDuplicates(tt)
  local newtable = {}
  for ii,xx in ipairs(tt) do
    if (CountOccurencesTable(newtable, xx) == 0) then
      newtable[#newtable+1] = xx
    end
  end
  return newtable
end -- RemoveTableDuplicates()

function MultiSplitMediaItem(item, times) --splits 'item' up into a table of segments returned as 'items'
                                          --'times' is a table of all the split points
  table.sort(times)

  local item_pos = reaper.GetMediaItemInfo_Value(item, "D_POSITION")
  local item_end = reaper.GetMediaItemInfo_Value(item, "D_LENGTH") + item_pos

  -- create array then reserve some space in array
  local items = {}

  -- add 'item' to 'items' array
  table.insert(items, item)

  -- for each time in times array do...
  for i, time in ipairs(times) do

    if time > item_end then break end

    if time > item_pos and time < item_end then
      --Msg( 'i = ' .. i .. " --- >".. time )

      if item then
        -- store item so we can split it next time around
        item = reaper.SplitMediaItem(item, time)

        -- add resulting item to array
        table.insert(items, item)
      end

    end

  end

  -- return 'items' array
  return items

end

-- CSV to Table
-- http://lua-users.org/wiki/LuaCsv
function ParseCSVLine (line,sep)
  local res = {}
  local pos = 1
   sep = sep or ','
  while true do
    local c = string.sub(line,pos,pos)
    if (c == "") then break end
    if (c == '"') then
      -- quoted value (ignore separator within)
      local txt = ""
      repeat
        local startp,endp = string.find(line,'^%b""',pos)
        txt = txt..string.sub(line,startp+1,endp-1)
        pos = endp + 1
        c = string.sub(line,pos,pos)
        if (c == '"') then txt = txt..'"' end
        -- check first char AFTER quoted string, if it is another
        -- quoted string without separator, then append it
        -- this is the way to "escape" the quote char in a quote. example:
        -- value1,"blub""blip""boing",value3 will result in blub"blip"boing for the middle
      until (c ~= '"')
      table.insert(res,txt)
      assert(c == sep or c == "")
      pos = pos + 1
    else
      -- no quotes used, just look for the first separator
      local startp,endp = string.find(line,sep,pos)
      if (startp) then
        table.insert(res,string.sub(line,pos,startp-1))
        pos = endp + 1
      else
        -- no separator found -> use rest of string and terminate
        table.insert(res,string.sub(line,pos))
        break
      end
    end
  end
  return res
end

function MeasureToSecond( measure, bpm, beat_per_measure, unit )
  local beat_speed = 1 / bpm * 60
  local beat_measure_count = measure[1] * beat_per_measure - beat_per_measure
  local beat_solo_count = measure[2] - 1
  local beat_incomplete = measure[3] * beat_speed / 100
  local beat_total = beat_measure_count + beat_solo_count + beat_incomplete
  return beat_total * beat_speed / ( unit / 4 )
end

function read_lines(filepath)

  local lines = {}

  local f = io.input(filepath)
  repeat

    s = f:read ("*l") -- read one line

    if s then -- if not end of file (EOF)
      table.insert(lines, ParseCSVLine( s, ',' ))
    end

  until not s -- until end of file

  f:close()

  return lines --'lines' is a table of values of table 'line' so like a excel array of csv values

end

function DeleteItems( items )

  for i, item in ipairs( items ) do
    local track = reaper.GetMediaItemTrack( item )
    reaper.DeleteTrackMediaItem( track, item )
  end

end

function SetItemFromBPMToBPM( item, bpm_source, bpm_target )

  local take = reaper.GetActiveTake(item)
  
  local take_type = reaper.TakeIsMIDI( take )
  
  if take_type then goto Midi 
  end 
  do
    local rate = bpm_target / bpm_source
   
    if take then 
      reaper.SetMediaItemTakeInfo_Value(take, "D_PLAYRATE", rate)
    end

    local item_len = reaper.GetMediaItemInfo_Value(item, "D_LENGTH")
    reaper.SetMediaItemInfo_Value(item, "D_LENGTH", item_len * ( bpm_source / bpm_target ) )
  
    return rate
  end
::Midi::  

end
--------------------------------------------------------- END OF UTILITIES

-- Main function
function Main() --Start running here

  master_tempo = reaper.Master_GetTempo() -- Note: Not current

  for i, item in ipairs(init_sel_items) do --In simple instance only one item selected

    local take = reaper.GetActiveTake( item ) --get active take for the selected media 

    local csv_exists = false
    local csv_filename = nil

    if take and not reaper.TakeIsMIDI( take ) then --check it is not MIDI media
      local source = reaper.GetMediaItemTake_Source( take ) --if not MIDI then get the wav or mp3 etc
      local filename = reaper.GetMediaSourceFileName( source, "" ) --Get file name of source media
      csv_filename = filename:gsub("%.(.+)", '.csv') --Change extension to csv
      csv_exists = reaper.file_exists( csv_filename ) --Check if csv file with same name exists
    end
    
    if take and reaper.TakeIsMIDI( take ) then --check it is MIDI media
      os_type = reaper.GetOS()
      sep = string.match(os_type, "Win") and "\\" or "/"
      inipath = reaper.get_ini_file()
      local item_source = reaper.GetMediaItemTake_Source( take )
      local item_filename = reaper.GetMediaSourceFileName( item_source, "" )
      local take_name = reaper.GetTakeName(take)
      --take_name_short = string.match(take_name, "- (.*)") --- not needed done by Script: ReaTrak set midi take name to filename.lua
      retval, midi_location = reaper.BR_Win32_GetPrivateProfileString("reaper", "importpath", "", inipath)
      local filename = tostring(midi_location) .. sep ..(take_name)
      csv_filename = filename:gsub("%.(.+)", '.csv') --Change extension to csv
      csv_exists = reaper.file_exists( csv_filename ) --Check if csv file with same name exists
    end    

    if not csv_exists then --If csv file is not found in the same directory then ask to import it
      csv_exists, csv_filename = reaper.GetUserFileNameForRead("", "Import items and regions from CSV", "csv")
    end

    if not csv_exists then break end --no luck finding a csv so can't do anything

    local lines = read_lines(csv_filename) --gets csv file data in table 'lines'

    local splits = {} 

    local bpm = tonumber(csv_filename:match('(%d+)bpm')) or 120 -- BPM From file name or default 120

    --local beats, unit = csv_filename:match('bpm[ |_](%d+)-(%d+)') --beats and unit from file name
    local beats, unit = csv_filename:match('(%d+)-(%d+)') --beats and unit from file name
    beats = tonumber( beats )
    unit = tonumber( unit )
    if not beats then beats = 4 end --default to beats = 4
    if not unit then unit = 4 end --default to ubits = 4

    local measure_count = lines[#lines][4] or 4 -- Simply taken from last entry in CSV, assuming it is last position

    SetItemFromBPMToBPM( item, bpm, master_tempo )
    local ratio = bpm / master_tempo

    local item_pos = reaper.GetMediaItemInfo_Value( item, "D_POSITION")
    local offset = ( ( MeasureToSecond( ParseCSVLine( lines[2][3], '%.' ), bpm, beats, unit ) + item_pos ) * ratio ) or 0

    for j, line in ipairs( lines ) do

      if line[1]:find('R') then -- If line is Region and not header or marker

        for column = 3, 5 do
          line[column] = ( MeasureToSecond( ParseCSVLine( line[column], '%.' ), bpm, beats, unit ) + item_pos ) * ratio
        end

        line[3] = line[3] - offset + item_pos -- negative marker compensation (for negative project start) and absolute
                                              --time (CSV is from proj start)
        line[4] = line[4] - offset + item_pos
        -- Insert split position to splits table as start1,end1,start2,end2,start3,end3 ... etc
        table.insert(splits, line[3])
        table.insert(splits, line[4])

      end

    end
        splits = RemoveTableDuplicates(splits) --end1=start2 and end2=start3 etc so remove the duplicates
        new_items = MultiSplitMediaItem(item, splits) --gets table of all the new item segments
    
    delete_items = {}

    for z, new_item in ipairs( new_items ) do

      local take = reaper.GetActiveTake( new_item )
      if take then
        local item_pos = reaper.GetMediaItemInfo_Value( new_item, "D_POSITION")
        local item_end = reaper.GetMediaItemInfo_Value( new_item, "D_LENGTH") + item_pos
        local take_name = reaper.GetTakeName( take )

        local set = false
        --Msg("")
        for k, line in ipairs( lines ) do

          set = false

          if line[1]:find('R') then -- If line is Region and not header or marker
            -- Msg(line[1])
            -- Msg(item_pos .. '=' .. line[3])
            if item_pos == line[3] then

-- check here which of the new_items created by the split correspond to regions and assign color from region to them
        local hex_color = line[6]
        hex = hex_color:gsub("#","") --if # sign is used in front of color code delete it
        local R = tonumber("0x"..hex:sub(1,2)) --Color codes are in reverse order in line table, B is last two digits
        local G = tonumber("0x"..hex:sub(3,4)) --so separate out the R G B components into 2 char strings and get
        local B = tonumber("0x"..hex:sub(5,6)) --decimal value for each component separately
        --color_int = (R + 256 * G + 65536 * B)|16777216  --recombine the components in right order R is least
                                                        --significant, G stays in middle and B is most significant
                                                        --OR with 0x1000000 (16777216 in decimal) to add 1 as 7th hex
                                                        --digit
        --reaper.SetMediaItemInfo_Value(new_item, "I_CUSTOMCOLOR", color_int)
        reaper.SetMediaItemInfo_Value(new_item, "I_CUSTOMCOLOR", reaper.ColorToNative( R, G, B ) |0x1000000) -- Works with Mac Color
        
-- now name the new items
              if replace_name then
                local retval, a_take_name = reaper.GetSetMediaItemTakeInfo_String(take, "P_NAME", line[2], 1)
              else
                local retval, a_take_name = reaper.GetSetMediaItemTakeInfo_String(take, "P_NAME", take_name .. "-" .. line[2], 1)
              end
              set = true
              break
            end

          end

        end

        if not set then -- for the last items
          table.insert( delete_items, new_item )
        end

      end

    end

    if delete_items then
      DeleteItems( delete_items )
    end

  end

end


-- INIT

-- See if there is items selected
count_sel_items = reaper.CountSelectedMediaItems(0)

if count_sel_items > 0 then

  reaper.PreventUIRefresh(1)

  reaper.Undo_BeginBlock() -- Begining of the undo block. Leave it at the top of your main function.

  reaper.ClearConsole()

  init_sel_items = {}
  SaveSelectedItems(init_sel_items)

  Main()

  reaper.Undo_EndBlock("ReaTrak Split to regions CSV", -1) -- End of the undo block. Leave it at the bottom of your main function.

  reaper.UpdateArrange()

  reaper.PreventUIRefresh(-1)

end
